@@ -42,7 +42,6 @@
 #include "ddk/wdm.h"
 
 #include "wine/asm.h"
-#include "wine/library.h"
 #include "wine/unicode.h"
 #include "kernel_private.h"
 
@@ -50,6 +49,9 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(sync);
 
+static const struct _KUSER_SHARED_DATA *user_shared_data = (struct _KUSER_SHARED_DATA *)0x7ffe0000;
+
+
 static void get_create_object_attributes( OBJECT_ATTRIBUTES *attr, UNICODE_STRING *nameW,
                                           SECURITY_ATTRIBUTES *sa, const WCHAR *name )
 {
@@ -101,44 +103,50 @@ static HANDLE normalize_handle_if_console(HANDLE handle)
     return handle;
 }
 
+/******************************************************************************
+ *           GetTickCount64       (KERNEL32.@)
+ */
+ULONGLONG WINAPI DECLSPEC_HOTPATCH GetTickCount64(void)
+{
+    ULONG high, low;
+
+    do
+    {
+        high = user_shared_data->u.TickCount.High1Time;
+        low = user_shared_data->u.TickCount.LowPart;
+    }
+    while (high != user_shared_data->u.TickCount.High2Time);
+    /* note: we ignore TickCountMultiplier */
+    return (ULONGLONG)high << 32 | low;
+}
+
+/***********************************************************************
+ *           GetTickCount       (KERNEL32.@)
+ */
+DWORD WINAPI DECLSPEC_HOTPATCH GetTickCount(void)
+{
+    /* note: we ignore TickCountMultiplier */
+    return user_shared_data->u.TickCount.LowPart;
+}
+
 /***********************************************************************
  *           RegisterWaitForSingleObject   (KERNEL32.@)
  */
-BOOL WINAPI RegisterWaitForSingleObject(PHANDLE phNewWaitObject, HANDLE hObject,
-                WAITORTIMERCALLBACK Callback, PVOID Context,
-                ULONG dwMilliseconds, ULONG dwFlags)
+BOOL WINAPI RegisterWaitForSingleObject( HANDLE *wait, HANDLE object, WAITORTIMERCALLBACK callback,
+                                         void *context, ULONG timeout, ULONG flags )
 {
-    NTSTATUS status;
+    TRACE( "%p %p %p %p %d %d\n", wait, object, callback, context, timeout, flags );
 
-    TRACE("%p %p %p %p %d %d\n",
-          phNewWaitObject,hObject,Callback,Context,dwMilliseconds,dwFlags);
-
-    hObject = normalize_handle_if_console(hObject);
-    status = RtlRegisterWait( phNewWaitObject, hObject, Callback, Context, dwMilliseconds, dwFlags );
-    if (status != STATUS_SUCCESS)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
-    return TRUE;
+    object = normalize_handle_if_console( object );
+    return set_ntstatus( RtlRegisterWait( wait, object, callback, context, timeout, flags ));
 }
 
 /***********************************************************************
  *           UnregisterWait   (KERNEL32.@)
  */
-BOOL WINAPI UnregisterWait( HANDLE WaitHandle ) 
+BOOL WINAPI UnregisterWait( HANDLE handle )
 {
-    NTSTATUS status;
-
-    TRACE("%p\n",WaitHandle);
-
-    status = RtlDeregisterWait( WaitHandle );
-    if (status != STATUS_SUCCESS)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
-    return TRUE;
+    return set_ntstatus( RtlDeregisterWait( handle ));
 }
 
 /***********************************************************************
@@ -311,16 +319,10 @@ HANDLE WINAPI OpenJobObjectW( DWORD access, BOOL inherit, LPCWSTR name )
     HANDLE ret;
     UNICODE_STRING nameW;
     OBJECT_ATTRIBUTES attr;
-    NTSTATUS status;
 
     if (!get_open_object_attributes( &attr, &nameW, inherit, name )) return 0;
 
-    status = NtOpenJobObject( &ret, access, &attr );
-    if (status != STATUS_SUCCESS)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return 0;
-    }
+    if (!set_ntstatus( NtOpenJobObject( &ret, access, &attr ))) return 0;
     return ret;
 }
 
@@ -346,9 +348,7 @@ HANDLE WINAPI OpenJobObjectA( DWORD access, BOOL inherit, LPCSTR name )
  */
 BOOL WINAPI TerminateJobObject( HANDLE job, UINT exit_code )
 {
-    NTSTATUS status = NtTerminateJobObject( job, exit_code );
-    if (status) SetLastError( RtlNtStatusToDosError(status) );
-    return !status;
+    return set_ntstatus( NtTerminateJobObject( job, exit_code ));
 }
 
 /******************************************************************************
@@ -357,9 +357,7 @@ BOOL WINAPI TerminateJobObject( HANDLE job, UINT exit_code )
 BOOL WINAPI QueryInformationJobObject( HANDLE job, JOBOBJECTINFOCLASS class, LPVOID info,
                                        DWORD len, DWORD *ret_len )
 {
-    NTSTATUS status = NtQueryInformationJobObject( job, class, info, len, ret_len );
-    if (status) SetLastError( RtlNtStatusToDosError(status) );
-    return !status;
+    return set_ntstatus( NtQueryInformationJobObject( job, class, info, len, ret_len ));
 }
 
 /******************************************************************************
@@ -367,9 +365,7 @@ BOOL WINAPI QueryInformationJobObject( HANDLE job, JOBOBJECTINFOCLASS class, LPV
  */
 BOOL WINAPI SetInformationJobObject( HANDLE job, JOBOBJECTINFOCLASS class, LPVOID info, DWORD len )
 {
-    NTSTATUS status = NtSetInformationJobObject( job, class, info, len );
-    if (status) SetLastError( RtlNtStatusToDosError(status) );
-    return !status;
+    return set_ntstatus( NtSetInformationJobObject( job, class, info, len ));
 }
 
 /******************************************************************************
@@ -377,29 +373,7 @@ BOOL WINAPI SetInformationJobObject( HANDLE job, JOBOBJECTINFOCLASS class, LPVOI
  */
 BOOL WINAPI AssignProcessToJobObject( HANDLE job, HANDLE process )
 {
-    NTSTATUS status = NtAssignProcessToJobObject( job, process );
-    if (status) SetLastError( RtlNtStatusToDosError(status) );
-    return !status;
-}
-
-/******************************************************************************
- *		IsProcessInJob (KERNEL32.@)
- */
-BOOL WINAPI IsProcessInJob( HANDLE process, HANDLE job, PBOOL result )
-{
-    NTSTATUS status = NtIsProcessInJob( process, job );
-    switch(status)
-    {
-    case STATUS_PROCESS_IN_JOB:
-        *result = TRUE;
-        return TRUE;
-    case STATUS_PROCESS_NOT_IN_JOB:
-        *result = FALSE;
-        return TRUE;
-    default:
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
+    return set_ntstatus( NtAssignProcessToJobObject( job, process ));
 }
 
 
@@ -566,18 +540,10 @@ BOOL WINAPI WaitNamedPipeA (LPCSTR name, DWORD nTimeOut)
 BOOL WINAPI GetNamedPipeClientProcessId( HANDLE pipe, ULONG *id )
 {
     IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
 
-    TRACE( "%p %p\n", pipe, id );
-
-    status = NtFsControlFile( pipe, NULL, NULL, NULL, &iosb, FSCTL_PIPE_GET_CONNECTION_ATTRIBUTE,
-                              (void *)"ClientProcessId", sizeof("ClientProcessId"), id, sizeof(*id) );
-    if (status)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
-    return TRUE;
+    return set_ntstatus( NtFsControlFile( pipe, NULL, NULL, NULL, &iosb,
+                                          FSCTL_PIPE_GET_CONNECTION_ATTRIBUTE, (void *)"ClientProcessId",
+                                          sizeof("ClientProcessId"), id, sizeof(*id) ));
 }
 
 /***********************************************************************
@@ -586,18 +552,10 @@ BOOL WINAPI GetNamedPipeClientProcessId( HANDLE pipe, ULONG *id )
 BOOL WINAPI GetNamedPipeServerProcessId( HANDLE pipe, ULONG *id )
 {
     IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
 
-    TRACE( "%p, %p\n", pipe, id );
-
-    status = NtFsControlFile( pipe, NULL, NULL, NULL, &iosb, FSCTL_PIPE_GET_CONNECTION_ATTRIBUTE,
-                              (void *)"ServerProcessId", sizeof("ServerProcessId"), id, sizeof(*id) );
-    if (status)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
-    return TRUE;
+    return set_ntstatus( NtFsControlFile( pipe, NULL, NULL, NULL, &iosb,
+                                          FSCTL_PIPE_GET_CONNECTION_ATTRIBUTE, (void *)"ServerProcessId",
+                                          sizeof("ServerProcessId"), id, sizeof(*id) ));
 }
 
 /***********************************************************************
@@ -650,65 +608,6 @@ BOOL WINAPI GetNamedPipeHandleStateA(
     return ret;
 }
 
-/***********************************************************************
- *           GetNamedPipeHandleStateW  (KERNEL32.@)
- */
-BOOL WINAPI GetNamedPipeHandleStateW(
-    HANDLE hNamedPipe, LPDWORD lpState, LPDWORD lpCurInstances,
-    LPDWORD lpMaxCollectionCount, LPDWORD lpCollectDataTimeout,
-    LPWSTR lpUsername, DWORD nUsernameMaxSize)
-{
-    IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
-
-    FIXME("%p %p %p %p %p %p %d: semi-stub\n", hNamedPipe, lpState, lpCurInstances,
-          lpMaxCollectionCount, lpCollectDataTimeout, lpUsername, nUsernameMaxSize);
-
-    if (lpMaxCollectionCount)
-        *lpMaxCollectionCount = 0;
-
-    if (lpCollectDataTimeout)
-        *lpCollectDataTimeout = 0;
-
-    if (lpUsername && nUsernameMaxSize)
-    {
-        const char *username = wine_get_user_name();
-        int len = MultiByteToWideChar(CP_UNIXCP, 0, username, -1, lpUsername, nUsernameMaxSize);
-        if (!len) *lpUsername = 0;
-    }
-
-    if (lpState)
-    {
-        FILE_PIPE_INFORMATION fpi;
-        status = NtQueryInformationFile(hNamedPipe, &iosb, &fpi, sizeof(fpi),
-                                        FilePipeInformation);
-        if (status)
-        {
-            SetLastError( RtlNtStatusToDosError(status) );
-            return FALSE;
-        }
-
-        *lpState = (fpi.ReadMode ? PIPE_READMODE_MESSAGE : PIPE_READMODE_BYTE) |
-                   (fpi.CompletionMode ? PIPE_NOWAIT : PIPE_WAIT);
-    }
-
-    if (lpCurInstances)
-    {
-        FILE_PIPE_LOCAL_INFORMATION fpli;
-        status = NtQueryInformationFile(hNamedPipe, &iosb, &fpli, sizeof(fpli),
-                                        FilePipeLocalInformation);
-        if (status)
-        {
-            SetLastError( RtlNtStatusToDosError(status) );
-            return FALSE;
-        }
-
-        *lpCurInstances = fpli.CurrentInstances;
-    }
-
-    return TRUE;
-}
-
 /***********************************************************************
  *           CallNamedPipeA  (KERNEL32.@)
  */
@@ -739,75 +638,6 @@ BOOL WINAPI CallNamedPipeA(
     return ret;
 }
 
-/******************************************************************
- *		CreatePipe (KERNEL32.@)
- *
- */
-BOOL WINAPI CreatePipe( PHANDLE hReadPipe, PHANDLE hWritePipe,
-                        LPSECURITY_ATTRIBUTES sa, DWORD size )
-{
-    static unsigned     index /* = 0 */;
-    WCHAR               name[64];
-    HANDLE              hr, hw;
-    unsigned            in_index = index;
-    UNICODE_STRING      nt_name;
-    OBJECT_ATTRIBUTES   attr;
-    NTSTATUS            status;
-    IO_STATUS_BLOCK     iosb;
-    LARGE_INTEGER       timeout;
-
-    *hReadPipe = *hWritePipe = INVALID_HANDLE_VALUE;
-
-    attr.Length                   = sizeof(attr);
-    attr.RootDirectory            = 0;
-    attr.ObjectName               = &nt_name;
-    attr.Attributes               = OBJ_CASE_INSENSITIVE |
-                                    ((sa && sa->bInheritHandle) ? OBJ_INHERIT : 0);
-    attr.SecurityDescriptor       = sa ? sa->lpSecurityDescriptor : NULL;
-    attr.SecurityQualityOfService = NULL;
-
-    if (!size) size = 4096;
-
-    timeout.QuadPart = (ULONGLONG)NMPWAIT_USE_DEFAULT_WAIT * -10000;
-    /* generate a unique pipe name (system wide) */
-    do
-    {
-        static const WCHAR nameFmt[] = { '\\','?','?','\\','p','i','p','e',
-         '\\','W','i','n','3','2','.','P','i','p','e','s','.','%','0','8','l',
-         'u','.','%','0','8','u','\0' };
-
-        snprintfW(name, ARRAY_SIZE(name), nameFmt, GetCurrentProcessId(), ++index);
-        RtlInitUnicodeString(&nt_name, name);
-        status = NtCreateNamedPipeFile(&hr, GENERIC_READ | FILE_WRITE_ATTRIBUTES | SYNCHRONIZE,
-                                       &attr, &iosb, FILE_SHARE_WRITE, FILE_OVERWRITE_IF,
-                                       FILE_SYNCHRONOUS_IO_NONALERT,
-                                       FALSE, FALSE, FALSE, 
-                                       1, size, size, &timeout);
-        if (status)
-        {
-            SetLastError( RtlNtStatusToDosError(status) );
-            hr = INVALID_HANDLE_VALUE;
-        }
-    } while (hr == INVALID_HANDLE_VALUE && index != in_index);
-    /* from completion sakeness, I think system resources might be exhausted before this happens !! */
-    if (hr == INVALID_HANDLE_VALUE) return FALSE;
-
-    status = NtOpenFile(&hw, GENERIC_WRITE | FILE_READ_ATTRIBUTES | SYNCHRONIZE, &attr, &iosb, 0,
-                        FILE_SYNCHRONOUS_IO_NONALERT | FILE_NON_DIRECTORY_FILE);
-
-    if (status) 
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        NtClose(hr);
-        return FALSE;
-    }
-
-    *hReadPipe = hr;
-    *hWritePipe = hw;
-    return TRUE;
-}
-
-
 /******************************************************************************
  * CreateMailslotA [KERNEL32.@]
  *
@@ -861,7 +691,6 @@ HANDLE WINAPI CreateMailslotW( LPCWSTR lpName, DWORD nMaxMessageSize,
     UNICODE_STRING nameW;
     LARGE_INTEGER timeout;
     IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
 
     TRACE("%s %d %d %p\n", debugstr_w(lpName),
           nMaxMessageSize, lReadTimeout, sa);
@@ -891,13 +720,9 @@ HANDLE WINAPI CreateMailslotW( LPCWSTR lpName, DWORD nMaxMessageSize,
     else
         timeout.QuadPart = ((LONGLONG)0x7fffffff << 32) | 0xffffffff;
 
-    status = NtCreateMailslotFile( &handle, GENERIC_READ | SYNCHRONIZE, &attr,
-                                   &iosb, 0, 0, nMaxMessageSize, &timeout );
-    if (status)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
+    if (!set_ntstatus( NtCreateMailslotFile( &handle, GENERIC_READ | SYNCHRONIZE, &attr,
+                                             &iosb, 0, 0, nMaxMessageSize, &timeout )))
         handle = INVALID_HANDLE_VALUE;
-    }
 
     RtlFreeUnicodeString( &nameW );
     return handle;
@@ -926,19 +751,13 @@ BOOL WINAPI GetMailslotInfo( HANDLE hMailslot, LPDWORD lpMaxMessageSize,
 {
     FILE_MAILSLOT_QUERY_INFORMATION info;
     IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
 
     TRACE("%p %p %p %p %p\n",hMailslot, lpMaxMessageSize,
           lpNextSize, lpMessageCount, lpReadTimeout);
 
-    status = NtQueryInformationFile( hMailslot, &iosb, &info, sizeof info,
-                                     FileMailslotQueryInformation );
-
-    if( status != STATUS_SUCCESS )
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
+    if (!set_ntstatus( NtQueryInformationFile( hMailslot, &iosb, &info, sizeof info,
+                                               FileMailslotQueryInformation )))
         return FALSE;
-    }
 
     if( lpMaxMessageSize )
         *lpMaxMessageSize = info.MaximumMessageSize;
@@ -974,7 +793,6 @@ BOOL WINAPI SetMailslotInfo( HANDLE hMailslot, DWORD dwReadTimeout)
 {
     FILE_MAILSLOT_SET_INFORMATION info;
     IO_STATUS_BLOCK iosb;
-    NTSTATUS status;
 
     TRACE("%p %d\n", hMailslot, dwReadTimeout);
 
@@ -982,124 +800,20 @@ BOOL WINAPI SetMailslotInfo( HANDLE hMailslot, DWORD dwReadTimeout)
         info.ReadTimeout.QuadPart = (ULONGLONG)dwReadTimeout * -10000;
     else
         info.ReadTimeout.QuadPart = ((LONGLONG)0x7fffffff << 32) | 0xffffffff;
-    status = NtSetInformationFile( hMailslot, &iosb, &info, sizeof info,
-                                   FileMailslotSetInformation );
-    if( status != STATUS_SUCCESS )
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return FALSE;
-    }
-    return TRUE;
+    return set_ntstatus( NtSetInformationFile( hMailslot, &iosb, &info, sizeof info,
+                                               FileMailslotSetInformation ));
 }
 
 
 /******************************************************************************
  *		BindIoCompletionCallback (KERNEL32.@)
  */
-BOOL WINAPI BindIoCompletionCallback( HANDLE FileHandle, LPOVERLAPPED_COMPLETION_ROUTINE Function, ULONG Flags)
+BOOL WINAPI BindIoCompletionCallback( HANDLE handle, LPOVERLAPPED_COMPLETION_ROUTINE func, ULONG flags )
 {
-    NTSTATUS status;
-
-    TRACE("(%p, %p, %d)\n", FileHandle, Function, Flags);
-
-    status = RtlSetIoCompletionCallback( FileHandle, (PRTL_OVERLAPPED_COMPLETION_ROUTINE)Function, Flags );
-    if (status == STATUS_SUCCESS) return TRUE;
-    SetLastError( RtlNtStatusToDosError(status) );
-    return FALSE;
+    return set_ntstatus( RtlSetIoCompletionCallback( handle, (PRTL_OVERLAPPED_COMPLETION_ROUTINE)func, flags ));
 }
 
 
-/***********************************************************************
- *           CreateMemoryResourceNotification   (KERNEL32.@)
- */
-HANDLE WINAPI CreateMemoryResourceNotification(MEMORY_RESOURCE_NOTIFICATION_TYPE type)
-{
-    static const WCHAR lowmemW[] =
-        {'\\','K','e','r','n','e','l','O','b','j','e','c','t','s',
-         '\\','L','o','w','M','e','m','o','r','y','C','o','n','d','i','t','i','o','n',0};
-    static const WCHAR highmemW[] =
-        {'\\','K','e','r','n','e','l','O','b','j','e','c','t','s',
-         '\\','H','i','g','h','M','e','m','o','r','y','C','o','n','d','i','t','i','o','n',0};
-    HANDLE ret;
-    UNICODE_STRING nameW;
-    OBJECT_ATTRIBUTES attr;
-    NTSTATUS status;
-
-    switch (type)
-    {
-    case LowMemoryResourceNotification:
-        RtlInitUnicodeString( &nameW, lowmemW );
-        break;
-    case HighMemoryResourceNotification:
-        RtlInitUnicodeString( &nameW, highmemW );
-        break;
-    default:
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return 0;
-    }
-
-    attr.Length                   = sizeof(attr);
-    attr.RootDirectory            = 0;
-    attr.ObjectName               = &nameW;
-    attr.Attributes               = 0;
-    attr.SecurityDescriptor       = NULL;
-    attr.SecurityQualityOfService = NULL;
-    status = NtOpenEvent( &ret, EVENT_ALL_ACCESS, &attr );
-    if (status != STATUS_SUCCESS)
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return 0;
-    }
-    return ret;
-}
-
-/***********************************************************************
- *          QueryMemoryResourceNotification   (KERNEL32.@)
- */
-BOOL WINAPI QueryMemoryResourceNotification(HANDLE handle, PBOOL state)
-{
-    switch (WaitForSingleObject( handle, 0 ))
-    {
-    case WAIT_OBJECT_0:
-        *state = TRUE;
-        return TRUE;
-    case WAIT_TIMEOUT:
-        *state = FALSE;
-        return TRUE;
-    }
-    SetLastError( ERROR_INVALID_PARAMETER );
-    return FALSE;
-}
-
-/***********************************************************************
- *           InitOnceBeginInitialize    (KERNEL32.@)
- */
-BOOL WINAPI InitOnceBeginInitialize( INIT_ONCE *once, DWORD flags, BOOL *pending, void **context )
-{
-    NTSTATUS status = RtlRunOnceBeginInitialize( once, flags, context );
-    if (status >= 0) *pending = (status == STATUS_PENDING);
-    else SetLastError( RtlNtStatusToDosError(status) );
-    return status >= 0;
-}
-
-/***********************************************************************
- *           InitOnceComplete    (KERNEL32.@)
- */
-BOOL WINAPI InitOnceComplete( INIT_ONCE *once, DWORD flags, void *context )
-{
-    NTSTATUS status = RtlRunOnceComplete( once, flags, context );
-    if (status != STATUS_SUCCESS) SetLastError( RtlNtStatusToDosError(status) );
-    return !status;
-}
-
-/***********************************************************************
- *           InitOnceExecuteOnce    (KERNEL32.@)
- */
-BOOL WINAPI InitOnceExecuteOnce( INIT_ONCE *once, PINIT_ONCE_FN func, void *param, void **context )
-{
-    return !RtlRunOnceExecuteOnce( once, (PRTL_RUN_ONCE_INIT_FN)func, param, context );
-}
-
 #ifdef __i386__
 
 /***********************************************************************
