@@ -238,39 +238,3 @@ DWORD WINAPI SetTermsrvAppInstallMode(BOOL bInstallMode)
     FIXME("(%d): stub\n", bInstallMode);
     return 0;
 }
-
-/***********************************************************************
- *           GetCurrentPackageId       (KERNEL32.@)
- */
-LONG WINAPI GetCurrentPackageId(UINT32 *len, BYTE *buffer)
-{
-    FIXME("(%p %p): stub\n", len, buffer);
-    return APPMODEL_ERROR_NO_PACKAGE;
-}
-
-/***********************************************************************
- *           GetCurrentPackageFamilyName       (KERNEL32.@)
- */
-LONG WINAPI GetCurrentPackageFamilyName(UINT32 *length, PWSTR name)
-{
-    FIXME("(%p %p): stub\n", length, name);
-    return APPMODEL_ERROR_NO_PACKAGE;
-}
-
-/***********************************************************************
- *           GetCurrentPackageFullName       (KERNEL32.@)
- */
-LONG WINAPI GetCurrentPackageFullName(UINT32 *length, PWSTR name)
-{
-    FIXME("(%p %p): stub\n", length, name);
-    return APPMODEL_ERROR_NO_PACKAGE;
-}
-
-/***********************************************************************
- *           GetPackageFullName       (KERNEL32.@)
- */
-LONG WINAPI GetPackageFullName(HANDLE process, UINT32 *length, PWSTR name)
-{
-    FIXME("(%p %p %p): stub\n", process, length, name);
-    return APPMODEL_ERROR_NO_PACKAGE;
-}
