@@ -27,6 +27,9 @@
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
+#ifdef HAVE_PWD_H
+# include <pwd.h>
+#endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -35,7 +38,6 @@
 #define WIN32_NO_STATUS
 #include "windef.h"
 #include "winternl.h"
-#include "wine/library.h"
 #include "wine/debug.h"
 #include "ntdll_misc.h"
 #include "winnt.h"
@@ -48,8 +50,6 @@ static const UNICODE_STRING null_str = { 0, 0, NULL };
 
 static const BOOL is_win64 = (sizeof(void *) > sizeof(int));
 
-static const WCHAR windows_dir[] = {'C',':','\\','w','i','n','d','o','w','s',0};
-
 static BOOL first_prefix_start;  /* first ever process start in this prefix? */
 
 static inline SIZE_T get_env_length( const WCHAR *env )
@@ -59,13 +59,6 @@ static inline SIZE_T get_env_length( const WCHAR *env )
     return end + 1 - env;
 }
 
-#ifdef __APPLE__
-extern char **__wine_get_main_environment(void);
-#else
-extern char **__wine_main_environ;
-static char **__wine_get_main_environment(void) { return __wine_main_environ; }
-#endif
-
 
 /***********************************************************************
  *           is_special_env_var
@@ -377,33 +370,57 @@ static void set_wow64_environment( WCHAR **env )
     static const WCHAR winehomedirW[] = {'W','I','N','E','H','O','M','E','D','I','R',0};
     static const WCHAR winedatadirW[] = {'W','I','N','E','D','A','T','A','D','I','R',0};
     static const WCHAR winebuilddirW[] = {'W','I','N','E','B','U','I','L','D','D','I','R',0};
+    static const WCHAR wineusernameW[] = {'W','I','N','E','U','S','E','R','N','A','M','E',0};
     static const WCHAR wineconfigdirW[] = {'W','I','N','E','C','O','N','F','I','G','D','I','R',0};
 
-    WCHAR buf[64];
+    WCHAR buf[256];
     UNICODE_STRING arch_strW = { sizeof(archW) - sizeof(WCHAR), sizeof(archW), archW };
     UNICODE_STRING arch6432_strW = { sizeof(arch6432W) - sizeof(WCHAR), sizeof(arch6432W), arch6432W };
     UNICODE_STRING valW = { 0, sizeof(buf), buf };
     OBJECT_ATTRIBUTES attr;
     UNICODE_STRING nameW;
-    const char *path;
+    const char *p;
+    const char *home = getenv( "HOME" );
+    const char *name = getenv( "USER" );
+    const char **dll_paths;
+    SIZE_T dll_path_maxlen;
     WCHAR *val;
     HANDLE hkey;
     DWORD i;
 
+    if (!home || !name)
+    {
+        struct passwd *pwd = getpwuid( getuid() );
+        if (pwd)
+        {
+            if (!home) home = pwd->pw_dir;
+            if (!name) name = pwd->pw_name;
+        }
+    }
+
     /* set the Wine paths */
 
-    set_wine_path_variable( env, winedatadirW, wine_get_data_dir() );
-    set_wine_path_variable( env, winehomedirW, getenv("HOME") );
-    set_wine_path_variable( env, winebuilddirW, wine_get_build_dir() );
-    set_wine_path_variable( env, wineconfigdirW, wine_get_config_dir() );
-    for (i = 0; (path = wine_dll_enum_load_path( i )); i++)
+    set_wine_path_variable( env, winedatadirW, data_dir );
+    set_wine_path_variable( env, winehomedirW, home );
+    set_wine_path_variable( env, winebuilddirW, build_dir );
+    set_wine_path_variable( env, wineconfigdirW, config_dir );
+    unix_funcs->get_dll_path( &dll_paths, &dll_path_maxlen );
+    for (i = 0; dll_paths[i]; i++)
     {
         NTDLL_swprintf( buf, winedlldirW, i );
-        set_wine_path_variable( env, buf, path );
+        set_wine_path_variable( env, buf, dll_paths[i] );
     }
     NTDLL_swprintf( buf, winedlldirW, i );
     set_wine_path_variable( env, buf, NULL );
 
+    /* set user name */
+
+    if (!name) name = "wine";
+    if ((p = strrchr( name, '/' ))) name = p + 1;
+    if ((p = strrchr( name, '\\' ))) name = p + 1;
+    ntdll_umbstowcs( name, strlen(name) + 1, buf, ARRAY_SIZE(buf) );
+    set_env_var( env, wineusernameW, buf );
+
     /* set the PROCESSOR_ARCHITECTURE variable */
 
     if (!RtlQueryEnvironmentVariable_U( *env, &arch6432_strW, &valW ))
@@ -756,8 +773,6 @@ static void set_library_wargv( char **argv, const UNICODE_STRING *image )
         total -= reslen;
     }
     wargv[argc] = NULL;
-
-    __wine_main_argc = argc;
     __wine_main_wargv = wargv;
 }
 
@@ -967,20 +982,69 @@ NTSTATUS WINAPI RtlQueryEnvironmentVariable_U(PWSTR env,
     return nts;
 }
 
+
+/******************************************************************
+ *		RtlQueryEnvironmentVariable   [NTDLL.@]
+ */
+NTSTATUS WINAPI RtlQueryEnvironmentVariable( WCHAR *env, const WCHAR *name, SIZE_T namelen,
+                                             WCHAR *value, SIZE_T value_length, SIZE_T *return_length )
+{
+    NTSTATUS nts = STATUS_VARIABLE_NOT_FOUND;
+    SIZE_T len = 0;
+    const WCHAR *var;
+
+    if (!namelen) return nts;
+
+    if (!env)
+    {
+        RtlAcquirePebLock();
+        var = NtCurrentTeb()->Peb->ProcessParameters->Environment;
+    }
+    else var = env;
+
+    var = ENV_FindVariable(var, name, namelen);
+    if (var != NULL)
+    {
+        len = wcslen(var);
+        if (len <= value_length)
+        {
+            memcpy(value, var, min(len + 1, value_length) * sizeof(WCHAR));
+            nts = STATUS_SUCCESS;
+        }
+        else
+        {
+            len++;
+            nts = STATUS_BUFFER_TOO_SMALL;
+        }
+    }
+    *return_length = len;
+
+    if (!env) RtlReleasePebLock();
+
+    return nts;
+}
+
 /******************************************************************
  *		RtlSetCurrentEnvironment        [NTDLL.@]
  *
  */
 void WINAPI RtlSetCurrentEnvironment(PWSTR new_env, PWSTR* old_env)
 {
+    WCHAR *prev;
+
     TRACE("(%p %p)\n", new_env, old_env);
 
     RtlAcquirePebLock();
 
-    if (old_env) *old_env = NtCurrentTeb()->Peb->ProcessParameters->Environment;
+    prev = NtCurrentTeb()->Peb->ProcessParameters->Environment;
     NtCurrentTeb()->Peb->ProcessParameters->Environment = new_env;
 
     RtlReleasePebLock();
+
+    if (old_env)
+        *old_env = prev;
+    else
+        RtlDestroyEnvironment( prev );
 }
 
 
@@ -1435,6 +1499,10 @@ void init_user_process_params( SIZE_T data_size )
     startup_info_t *info = NULL;
     RTL_USER_PROCESS_PARAMETERS *params = NULL;
     UNICODE_STRING curdir, dllpath, imagepath, cmdline, title, desktop, shellinfo, runtime;
+    int argc;
+    char **argv, **envp;
+
+    unix_funcs->get_main_args( &argc, &argv, &envp );
 
     if (!data_size)
     {
@@ -1442,13 +1510,13 @@ void init_user_process_params( SIZE_T data_size )
         WCHAR *env, curdir_buffer[MAX_PATH];
 
         NtCurrentTeb()->Peb->ProcessParameters = &initial_params;
-        initial_params.Environment = build_initial_environment( __wine_get_main_environment() );
+        initial_params.Environment = build_initial_environment( envp );
         curdir.Buffer = curdir_buffer;
         curdir.MaximumLength = sizeof(curdir_buffer);
         get_current_directory( &curdir );
         initial_params.CurrentDirectory.DosPath = curdir;
-        get_image_path( __wine_main_argv[0], &initial_params.ImagePathName );
-        set_library_wargv( __wine_main_argv, &initial_params.ImagePathName );
+        get_image_path( argv[0], &initial_params.ImagePathName );
+        set_library_wargv( argv, &initial_params.ImagePathName );
         build_command_line( __wine_main_wargv, &cmdline );
         LdrGetDllPath( initial_params.ImagePathName.Buffer, 0, &load_path, &dummy );
         RtlInitUnicodeString( &dllpath, load_path );
@@ -1462,6 +1530,7 @@ void init_user_process_params( SIZE_T data_size )
 
         params->Environment = env;
         NtCurrentTeb()->Peb->ProcessParameters = params;
+        RtlFreeUnicodeString( &initial_params.ImagePathName );
         RtlFreeUnicodeString( &cmdline );
         RtlReleasePath( load_path );
 
@@ -1535,7 +1604,7 @@ void init_user_process_params( SIZE_T data_size )
         else params->Environment[0] = 0;
     }
 
-    set_library_wargv( __wine_main_argv, NULL );
+    set_library_wargv( argv, NULL );
 
 done:
     RtlFreeHeap( GetProcessHeap(), 0, info );
