@@ -24,9 +24,6 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_IO_H
-# include <io.h>
-#endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -38,6 +35,7 @@
 #include "winternl.h"
 #include "ntdll_misc.h"
 #include "wine/server.h"
+#include "wine/exception.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(ntdll);
 
@@ -353,43 +351,39 @@ NTSTATUS WINAPI NtDuplicateObject( HANDLE source_process, HANDLE source,
                                    HANDLE dest_process, PHANDLE dest,
                                    ACCESS_MASK access, ULONG attributes, ULONG options )
 {
-    NTSTATUS ret;
-    SERVER_START_REQ( dup_handle )
-    {
-        req->src_process = wine_server_obj_handle( source_process );
-        req->src_handle  = wine_server_obj_handle( source );
-        req->dst_process = wine_server_obj_handle( dest_process );
-        req->access      = access;
-        req->attributes  = attributes;
-        req->options     = options;
+    return unix_funcs->NtDuplicateObject( source_process, source, dest_process,
+                                          dest, access, attributes, options );
+}
 
-        if (!(ret = wine_server_call( req )))
-        {
-            if (dest) *dest = wine_server_ptr_handle( reply->handle );
-            if (reply->closed && reply->self)
-            {
-                int fd = server_remove_fd_from_cache( source );
-                if (fd != -1) close( fd );
-            }
-        }
-    }
-    SERVER_END_REQ;
-    return ret;
+static LONG WINAPI invalid_handle_exception_handler( EXCEPTION_POINTERS *eptr )
+{
+    EXCEPTION_RECORD *rec = eptr->ExceptionRecord;
+    return (rec->ExceptionCode == EXCEPTION_INVALID_HANDLE) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH;
 }
 
 /* Everquest 2 / Pirates of the Burning Sea hooks NtClose, so we need a wrapper */
 NTSTATUS close_handle( HANDLE handle )
 {
-    NTSTATUS ret;
-    int fd = server_remove_fd_from_cache( handle );
+    NTSTATUS ret = unix_funcs->NtClose( handle );
 
-    SERVER_START_REQ( close_handle )
+    if (ret == STATUS_INVALID_HANDLE && handle && NtCurrentTeb()->Peb->BeingDebugged)
     {
-        req->handle = wine_server_obj_handle( handle );
-        ret = wine_server_call( req );
+        __TRY
+        {
+            EXCEPTION_RECORD record;
+            record.ExceptionCode    = EXCEPTION_INVALID_HANDLE;
+            record.ExceptionFlags   = 0;
+            record.ExceptionRecord  = NULL;
+            record.ExceptionAddress = NULL;
+            record.NumberParameters = 0;
+            RtlRaiseException( &record );
+        }
+        __EXCEPT(invalid_handle_exception_handler)
+        {
+        }
+        __ENDTRY
     }
-    SERVER_END_REQ;
-    if (fd != -1) close( fd );
+
     return ret;
 }
 
