@@ -30,9 +30,6 @@
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #endif
-#ifdef HAVE_IO_H
-# include <io.h>
-#endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -341,6 +338,31 @@ static NTSTATUS get_modem_status(int fd, DWORD* lpModemStat)
     return status;
 }
 
+static NTSTATUS get_properties(int fd, SERIAL_COMMPROP *prop)
+{
+    /* FIXME: get actual properties from the device */
+    memset( prop, 0, sizeof(*prop) );
+    prop->PacketLength       = 1;
+    prop->PacketVersion      = 1;
+    prop->ServiceMask       = SP_SERIALCOMM;
+    prop->MaxTxQueue        = 4096;
+    prop->MaxRxQueue        = 4096;
+    prop->MaxBaud           = BAUD_115200;
+    prop->ProvSubType       = PST_RS232;
+    prop->ProvCapabilities  = PCF_DTRDSR | PCF_PARITY_CHECK | PCF_RTSCTS | PCF_TOTALTIMEOUTS | PCF_INTTIMEOUTS;
+    prop->SettableParams    = SP_BAUD | SP_DATABITS | SP_HANDSHAKING |
+                              SP_PARITY | SP_PARITY_CHECK | SP_STOPBITS ;
+    prop->SettableBaud      = BAUD_075 | BAUD_110 | BAUD_134_5 | BAUD_150 |
+                              BAUD_300 | BAUD_600 | BAUD_1200 | BAUD_1800 | BAUD_2400 | BAUD_4800 |
+                              BAUD_9600 | BAUD_19200 | BAUD_38400 | BAUD_57600 | BAUD_115200 ;
+    prop->SettableData       = DATABITS_5 | DATABITS_6 | DATABITS_7 | DATABITS_8 ;
+    prop->SettableStopParity = STOPBITS_10 | STOPBITS_15 | STOPBITS_20 |
+                PARITY_NONE | PARITY_ODD |PARITY_EVEN | PARITY_MARK | PARITY_SPACE;
+    prop->CurrentTxQueue    = prop->MaxTxQueue;
+    prop->CurrentRxQueue    = prop->MaxRxQueue;
+    return STATUS_SUCCESS;
+}
+
 static NTSTATUS get_special_chars(int fd, SERIAL_CHARS* sc)
 {
     struct termios port;
@@ -929,7 +951,7 @@ static DWORD CALLBACK wait_for_event(LPVOID arg)
     async_commio *commio = arg;
     int fd, needs_close;
 
-    if (!server_get_unix_fd( commio->hDevice, FILE_READ_DATA | FILE_WRITE_DATA, &fd, &needs_close, NULL, NULL ))
+    if (!unix_funcs->server_get_unix_fd( commio->hDevice, FILE_READ_DATA | FILE_WRITE_DATA, &fd, &needs_close, NULL, NULL ))
     {
         serial_irq_info new_irq_info;
         DWORD new_mstat, dummy, cookie;
@@ -1119,7 +1141,7 @@ static inline NTSTATUS io_control(HANDLE hDevice,
 
     piosb->Information = 0;
 
-    if ((status = server_get_unix_fd( hDevice, access, &fd, &needs_close, &type, NULL )))
+    if ((status = unix_funcs->server_get_unix_fd( hDevice, access, &fd, &needs_close, &type, NULL )))
         goto error;
     if (type != FD_TYPE_SERIAL)
     {
@@ -1196,6 +1218,14 @@ static inline NTSTATUS io_control(HANDLE hDevice,
         }
         else status = STATUS_INVALID_PARAMETER;
         break;
+    case IOCTL_SERIAL_GET_PROPERTIES:
+        if (lpOutBuffer && nOutBufferSize == sizeof(SERIAL_COMMPROP))
+        {
+            if (!(status = get_properties(fd, lpOutBuffer)))
+                sz = sizeof(SERIAL_COMMPROP);
+        }
+        else status = STATUS_INVALID_PARAMETER;
+        break;
     case IOCTL_SERIAL_IMMEDIATE_CHAR:
         if (lpInBuffer && nInBufferSize == sizeof(CHAR))
             status = xmit_immediate(hDevice, fd, lpInBuffer);
