@@ -148,8 +148,9 @@ static void main_test(void)
     static const WCHAR dokW[] = {'d','o','k',0};
     WCHAR temppathW[MAX_PATH], pathW[MAX_PATH];
     struct test_input *test_input;
-    UNICODE_STRING pathU;
     DWORD len, written, read;
+    ULONG64 modified_value;
+    UNICODE_STRING pathU;
     LONG new_failures;
     char buffer[512];
     HANDLE okfile;
@@ -165,6 +166,11 @@ static void main_test(void)
     test_input->running_under_wine = !strcmp(winetest_platform, "wine");
     test_input->winetest_report_success = winetest_report_success;
     test_input->winetest_debug = winetest_debug;
+    test_input->process_id = GetCurrentProcessId();
+    test_input->teststr_offset = (SIZE_T)((BYTE *)&teststr - (BYTE *)NtCurrentTeb()->Peb->ImageBaseAddress);
+    test_input->modified_value = &modified_value;
+    modified_value = 0;
+
     memcpy(test_input->path, pathU.Buffer, len);
     res = DeviceIoControl(device, IOCTL_WINETEST_MAIN_TEST, test_input,
                           offsetof( struct test_input, path[len / sizeof(WCHAR)]),
@@ -172,6 +178,8 @@ static void main_test(void)
     ok(res, "DeviceIoControl failed: %u\n", GetLastError());
     ok(written == sizeof(new_failures), "got size %x\n", written);
 
+    todo_wine ok(modified_value == 0xdeadbeeffeedcafe, "Got unexpected value %#I64x.\n", modified_value);
+
     okfile = CreateFileW(pathW, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
     ok(okfile != INVALID_HANDLE_VALUE, "failed to create %s: %u\n", wine_dbgstr_w(pathW), GetLastError());
 
@@ -190,8 +198,8 @@ static void main_test(void)
 
 static void test_basic_ioctl(void)
 {
+    char inbuf[64], buf[32];
     DWORD written;
-    char buf[32];
     BOOL res;
 
     res = DeviceIoControl(device, IOCTL_WINETEST_BASIC_IOCTL, NULL, 0, buf,
@@ -199,6 +207,25 @@ static void test_basic_ioctl(void)
     ok(res, "DeviceIoControl failed: %u\n", GetLastError());
     ok(written == sizeof(teststr), "got size %d\n", written);
     ok(!strcmp(buf, teststr), "got '%s'\n", buf);
+
+    memset(buf, 0, sizeof(buf));
+    res = DeviceIoControl(device, IOCTL_WINETEST_BASIC_IOCTL, inbuf,
+            sizeof(inbuf), buf, 10, &written, NULL);
+    ok(res, "DeviceIoControl failed: %u\n", GetLastError());
+    ok(written == 10, "got size %d\n", written);
+    ok(!strcmp(buf, "Wine is no"), "got '%s'\n", buf);
+}
+
+static void test_mismatched_status_ioctl(void)
+{
+    DWORD written;
+    char buf[32];
+    BOOL res;
+
+    res = DeviceIoControl(device, IOCTL_WINETEST_MISMATCHED_STATUS, NULL, 0, buf,
+                          sizeof(buf), &written, NULL);
+    todo_wine ok(res, "DeviceIoControl failed: %u\n", GetLastError());
+    todo_wine ok(!strcmp(buf, teststr), "got '%s'\n", buf);
 }
 
 static void test_overlapped(void)
@@ -507,6 +534,7 @@ START_TEST(ntoskrnl)
     ok(device != INVALID_HANDLE_VALUE, "failed to open device: %u\n", GetLastError());
 
     test_basic_ioctl();
+    test_mismatched_status_ioctl();
     main_test();
     test_overlapped();
     test_load_driver(service2);
