@@ -20,6 +20,7 @@
 
 #define COBJMACROS
 #include "dshow.h"
+#include "wine/strmbase.h"
 #include "wine/test.h"
 
 static const WCHAR sink_id[] = {'I','n','p','u','t',0};
@@ -44,6 +45,12 @@ static ULONG get_refcount(void *iface)
     return IUnknown_Release(unknown);
 }
 
+static inline BOOL compare_media_types(const AM_MEDIA_TYPE *a, const AM_MEDIA_TYPE *b)
+{
+    return !memcmp(a, b, offsetof(AM_MEDIA_TYPE, pbFormat))
+            && !memcmp(a->pbFormat, b->pbFormat, a->cbFormat);
+}
+
 #define check_interface(a, b, c) check_interface_(__LINE__, a, b, c)
 static void check_interface_(unsigned int line, void *iface_ptr, REFIID iid, BOOL supported)
 {
@@ -178,12 +185,12 @@ static void test_aggregation(void)
     ULONG ref;
 
     filter = (IBaseFilter *)0xdeadbeef;
-    hr = CoCreateInstance(&CLSID_AviDest, &test_outer, CLSCTX_INPROC_SERVER,
+    hr = CoCreateInstance(&CLSID_SmartTee, &test_outer, CLSCTX_INPROC_SERVER,
             &IID_IBaseFilter, (void **)&filter);
     ok(hr == E_NOINTERFACE, "Got hr %#x.\n", hr);
     ok(!filter, "Got interface %p.\n", filter);
 
-    hr = CoCreateInstance(&CLSID_AviDest, &test_outer, CLSCTX_INPROC_SERVER,
+    hr = CoCreateInstance(&CLSID_SmartTee, &test_outer, CLSCTX_INPROC_SERVER,
             &IID_IUnknown, (void **)&unk);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ok(outer_ref == 1, "Got unexpected refcount %d.\n", outer_ref);
@@ -2005,30 +2012,6 @@ static SourceFilter* create_video_SourceFilter(void)
     return This;
 }
 
-static SourceFilter* create_audio_SourceFilter(void)
-{
-    SourceFilter *This = create_SourceFilter();
-    if (!This)
-        return NULL;
-    This->mediaType.majortype = MEDIATYPE_Audio;
-    This->mediaType.subtype = MEDIASUBTYPE_PCM;
-    This->mediaType.bFixedSizeSamples = FALSE;
-    This->mediaType.bTemporalCompression = FALSE;
-    This->mediaType.lSampleSize = 0;
-    This->mediaType.formattype = FORMAT_WaveFormatEx;
-    This->mediaType.pUnk = NULL;
-    This->mediaType.cbFormat = sizeof(WAVEFORMATEX);
-    This->mediaType.pbFormat = (BYTE*) &This->audioInfo;
-    This->audioInfo.wFormatTag = WAVE_FORMAT_PCM;
-    This->audioInfo.nChannels = 1;
-    This->audioInfo.nSamplesPerSec = 8000;
-    This->audioInfo.nAvgBytesPerSec = 16000;
-    This->audioInfo.nBlockAlign = 2;
-    This->audioInfo.wBitsPerSample = 16;
-    This->audioInfo.cbSize = 0;
-    return This;
-}
-
 static void test_smart_tee_filter_in_graph(IBaseFilter *smartTeeFilter, IPin *inputPin,
         IPin *capturePin, IPin *previewPin)
 {
@@ -2215,20 +2198,6 @@ static void test_smart_tee_filter(void)
     hr = IMemInputPin_ReceiveCanBlock(memInputPin);
     ok(hr == S_OK, "unexpected IMemInputPin_ReceiveCanBlock() = 0x%08x\n", hr);
 
-    hr = IPin_EnumMediaTypes(inputPin, &enumMediaTypes);
-    ok(SUCCEEDED(hr), "IPin_EnumMediaTypes() failed, hr=0x%08x\n", hr);
-    if (SUCCEEDED(hr)) {
-        AM_MEDIA_TYPE *mediaType = NULL;
-        hr = IEnumMediaTypes_Next(enumMediaTypes, 1, &mediaType, NULL);
-        ok(hr == S_FALSE, "the media types are non-empty\n");
-    }
-    IEnumMediaTypes_Release(enumMediaTypes);
-    enumMediaTypes = NULL;
-    hr = IPin_EnumMediaTypes(capturePin, &enumMediaTypes);
-    ok(hr == VFW_E_NOT_CONNECTED, "IPin_EnumMediaTypes() failed, hr=0x%08x\n", hr);
-    hr = IPin_EnumMediaTypes(previewPin, &enumMediaTypes);
-    ok(hr == VFW_E_NOT_CONNECTED, "IPin_EnumMediaTypes() failed, hr=0x%08x\n", hr);
-
     test_smart_tee_filter_in_graph(smartTeeFilter, inputPin, capturePin, previewPin);
 
 end:
@@ -2305,205 +2274,493 @@ static void test_unconnected_filter_state(void)
     ok(!ref, "Got outstanding refcount %d.\n", ref);
 }
 
-static HRESULT get_connected_filter_classid(IPin *pin, GUID *guid)
+struct testfilter
 {
-    IPin *connectedPin = NULL;
-    PIN_INFO connectedPinInfo;
-    HRESULT hr = IPin_ConnectedTo(pin, &connectedPin);
-    ok(SUCCEEDED(hr), "IPin_ConnectedTo() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
-    hr = IPin_QueryPinInfo(connectedPin, &connectedPinInfo);
-    ok(SUCCEEDED(hr), "IPin_QueryPinInfo() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
-    if (connectedPinInfo.pFilter) {
-        hr = IBaseFilter_GetClassID(connectedPinInfo.pFilter, guid);
-        ok(SUCCEEDED(hr), "IBaseFilter_GetClassID() failed, hr=0x%08x\n", hr);
-        IBaseFilter_Release(connectedPinInfo.pFilter);
+    struct strmbase_filter filter;
+    struct strmbase_source source;
+    struct strmbase_sink sink;
+    const AM_MEDIA_TYPE *sink_mt;
+    AM_MEDIA_TYPE source_mt;
+};
+
+static inline struct testfilter *impl_from_strmbase_filter(struct strmbase_filter *iface)
+{
+    return CONTAINING_RECORD(iface, struct testfilter, filter);
+}
+
+static struct strmbase_pin *testfilter_get_pin(struct strmbase_filter *iface, unsigned int index)
+{
+    struct testfilter *filter = impl_from_strmbase_filter(iface);
+    if (!index)
+        return &filter->source.pin;
+    else if (index == 1)
+        return &filter->sink.pin;
+    return NULL;
+}
+
+static void testfilter_destroy(struct strmbase_filter *iface)
+{
+    struct testfilter *filter = impl_from_strmbase_filter(iface);
+    strmbase_source_cleanup(&filter->source);
+    strmbase_sink_cleanup(&filter->sink);
+    strmbase_filter_cleanup(&filter->filter);
+}
+
+static const struct strmbase_filter_ops testfilter_ops =
+{
+    .filter_get_pin = testfilter_get_pin,
+    .filter_destroy = testfilter_destroy,
+};
+
+static HRESULT testsource_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *mt)
+{
+    return mt->bTemporalCompression ? S_OK : S_FALSE;
+}
+
+static HRESULT testsource_get_media_type(struct strmbase_pin *iface, unsigned int index, AM_MEDIA_TYPE *mt)
+{
+    struct testfilter *filter = impl_from_strmbase_filter(iface->filter);
+    if (!index)
+    {
+        CopyMediaType(mt, &filter->source_mt);
+        return S_OK;
     }
-end:
-    if (connectedPin)
-        IPin_Release(connectedPin);
+    return VFW_S_NO_MORE_ITEMS;
+}
+
+static void test_sink_allocator(IPin *pin)
+{
+    ALLOCATOR_PROPERTIES req_props = {1, 5000, 1, 0}, ret_props;
+    IMemAllocator *req_allocator, *ret_allocator;
+    IMemInputPin *input;
+    HRESULT hr;
+
+    IPin_QueryInterface(pin, &IID_IMemInputPin, (void **)&input);
+
+    hr = IMemInputPin_GetAllocatorRequirements(input, &ret_props);
+    ok(hr == E_NOTIMPL, "Got hr %#x.\n", hr);
+
+    hr = IMemInputPin_GetAllocator(input, &ret_allocator);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IMemInputPin_NotifyAllocator(input, ret_allocator, TRUE);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    IMemAllocator_Release(ret_allocator);
+
+    CoCreateInstance(&CLSID_MemoryAllocator, NULL, CLSCTX_INPROC_SERVER,
+            &IID_IMemAllocator, (void **)&req_allocator);
+
+    hr = IMemInputPin_NotifyAllocator(input, req_allocator, TRUE);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IMemInputPin_GetAllocator(input, &ret_allocator);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(ret_allocator == req_allocator, "Allocators didn't match.\n");
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    IMemAllocator_Release(ret_allocator);
+
+    hr = IMemAllocator_SetProperties(req_allocator, &req_props, &ret_props);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IMemAllocator_Commit(req_allocator);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    IMemAllocator_Release(req_allocator);
+    IMemInputPin_Release(input);
+}
+
+static HRESULT WINAPI testsource_AttemptConnection(struct strmbase_source *iface,
+        IPin *peer, const AM_MEDIA_TYPE *mt)
+{
+    HRESULT hr;
+
+    iface->pin.peer = peer;
+    IPin_AddRef(peer);
+    CopyMediaType(&iface->pin.mt, mt);
+
+    if (FAILED(hr = IPin_ReceiveConnection(peer, &iface->pin.IPin_iface, mt)))
+    {
+        ok(hr == VFW_E_TYPE_NOT_ACCEPTED, "Got hr %#x.\n", hr);
+        IPin_Release(peer);
+        iface->pin.peer = NULL;
+        FreeMediaType(&iface->pin.mt);
+    }
+
+    test_sink_allocator(peer);
+
     return hr;
 }
 
-static void test_audio_preview(ICaptureGraphBuilder2 *captureGraphBuilder, IGraphBuilder *graphBuilder,
-        SourceFilter *audioSource, IBaseFilter *nullRenderer)
+static const struct strmbase_source_ops testsource_ops =
 {
-    GUID clsid;
-    HRESULT hr = ICaptureGraphBuilder2_RenderStream(captureGraphBuilder, &PIN_CATEGORY_PREVIEW, &MEDIATYPE_Audio,
-                (IUnknown*)&audioSource->IBaseFilter_iface, NULL, nullRenderer);
-    ok(hr == VFW_S_NOPREVIEWPIN, "ICaptureGraphBuilder2_RenderStream() returned hr=0x%08x\n", hr);
-    hr = get_connected_filter_classid(&audioSource->IPin_iface, &clsid);
-    if (FAILED(hr))
-        return;
-    ok(IsEqualIID(&clsid, &CLSID_SmartTee), "unexpected connected filter %s\n",
-            wine_dbgstr_guid(&clsid));
+    .base.pin_query_accept = testsource_query_accept,
+    .base.pin_get_media_type = testsource_get_media_type,
+    .pfnAttemptConnection = testsource_AttemptConnection,
+};
+
+static HRESULT testsink_query_interface(struct strmbase_pin *iface, REFIID iid, void **out)
+{
+    struct testfilter *filter = impl_from_strmbase_filter(iface->filter);
+
+    if (IsEqualGUID(iid, &IID_IMemInputPin))
+        *out = &filter->sink.IMemInputPin_iface;
+    else
+        return E_NOINTERFACE;
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
 }
 
-static void test_audio_capture(ICaptureGraphBuilder2 *captureGraphBuilder, IGraphBuilder *graphBuilder,
-        SourceFilter *audioSource, IBaseFilter *nullRenderer)
+static HRESULT testsink_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *mt)
 {
-    GUID clsid;
-    HRESULT hr = ICaptureGraphBuilder2_RenderStream(captureGraphBuilder, &PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio,
-                (IUnknown*)&audioSource->IBaseFilter_iface, NULL, nullRenderer);
-    ok(hr == S_OK, "ICaptureGraphBuilder2_RenderStream() returned hr=0x%08x\n", hr);
-    hr = get_connected_filter_classid(&audioSource->IPin_iface, &clsid);
-    if (FAILED(hr))
-        return;
-    ok(IsEqualIID(&clsid, &CLSID_SmartTee), "unexpected connected filter %s\n",
-            wine_dbgstr_guid(&clsid));
+    struct testfilter *filter = impl_from_strmbase_filter(iface->filter);
+    if (filter->sink_mt && !compare_media_types(mt, filter->sink_mt))
+        return S_FALSE;
+    return S_OK;
 }
 
-static void test_video_preview(ICaptureGraphBuilder2 *captureGraphBuilder, IGraphBuilder *graphBuilder,
-        SourceFilter *videoSource, IBaseFilter *nullRenderer)
+static HRESULT testsink_get_media_type(struct strmbase_pin *iface, unsigned int index, AM_MEDIA_TYPE *mt)
 {
-    GUID clsid;
-    HRESULT hr = ICaptureGraphBuilder2_RenderStream(captureGraphBuilder, &PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video,
-                (IUnknown*)&videoSource->IBaseFilter_iface, NULL, nullRenderer);
-    ok(hr == VFW_S_NOPREVIEWPIN, "ICaptureGraphBuilder2_RenderStream() failed, hr=0x%08x\n", hr);
-    hr = get_connected_filter_classid(&videoSource->IPin_iface, &clsid);
-    if (FAILED(hr))
-        return;
-    ok(IsEqualIID(&clsid, &CLSID_SmartTee), "unexpected connected filter %s\n",
-            wine_dbgstr_guid(&clsid));
+    struct testfilter *filter = impl_from_strmbase_filter(iface->filter);
+    if (!index && filter->sink_mt)
+    {
+        CopyMediaType(mt, filter->sink_mt);
+        return S_OK;
+    }
+    return VFW_S_NO_MORE_ITEMS;
 }
 
-static void test_video_capture(ICaptureGraphBuilder2 *captureGraphBuilder, IGraphBuilder *graphBuilder,
-        SourceFilter *videoSource, IBaseFilter *nullRenderer)
+static HRESULT WINAPI testsink_Receive(struct strmbase_sink *iface, IMediaSample *sample)
 {
-    GUID clsid;
-    HRESULT hr = ICaptureGraphBuilder2_RenderStream(captureGraphBuilder, &PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video,
-                (IUnknown*)&videoSource->IBaseFilter_iface, NULL, nullRenderer);
-    ok(hr == S_OK, "ICaptureGraphBuilder2_RenderStream() failed, hr=0x%08x\n", hr);
-    hr = get_connected_filter_classid(&videoSource->IPin_iface, &clsid);
-    if (FAILED(hr))
-        return;
-    ok(IsEqualIID(&clsid, &CLSID_SmartTee), "unexpected connected filter %s\n",
-            wine_dbgstr_guid(&clsid));
+    return S_OK;
 }
 
-static void test_audio_smart_tee_filter_auto_insertion(
-        void (*test_function)(ICaptureGraphBuilder2 *cgb, IGraphBuilder *gb,
-                SourceFilter *audioSource, IBaseFilter *nullRenderer))
+static const struct strmbase_sink_ops testsink_ops =
 {
+    .base.pin_query_interface = testsink_query_interface,
+    .base.pin_query_accept = testsink_query_accept,
+    .base.pin_get_media_type = testsink_get_media_type,
+    .pfnReceive = testsink_Receive,
+};
+
+static void testfilter_init(struct testfilter *filter)
+{
+    static const GUID clsid = {0xabacab};
+    memset(filter, 0, sizeof(*filter));
+    strmbase_filter_init(&filter->filter, NULL, &clsid, &testfilter_ops);
+    strmbase_source_init(&filter->source, &filter->filter, L"source", &testsource_ops);
+    strmbase_sink_init(&filter->sink, &filter->filter, L"sink", &testsink_ops, NULL);
+}
+
+static void test_source_media_types(AM_MEDIA_TYPE req_mt, const AM_MEDIA_TYPE *source_mt, IPin *source)
+{
+    IEnumMediaTypes *enummt;
+    AM_MEDIA_TYPE *mts[3];
+    ULONG count;
     HRESULT hr;
-    ICaptureGraphBuilder2 *captureGraphBuilder = NULL;
-    IGraphBuilder *graphBuilder = NULL;
-    IBaseFilter *nullRenderer = NULL;
-    SourceFilter *audioSource = NULL;
 
-    hr = CoCreateInstance(&CLSID_CaptureGraphBuilder2, NULL, CLSCTX_INPROC_SERVER,
-            &IID_ICaptureGraphBuilder2, (void**)&captureGraphBuilder);
-    ok(SUCCEEDED(hr), "couldn't create capture graph builder, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    hr = IPin_EnumMediaTypes(source, &enummt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IEnumMediaTypes_Next(enummt, 3, mts, &count);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    todo_wine ok(count == 2, "Got %u types.\n", count);
+    ok(compare_media_types(mts[0], &req_mt), "Media types didn't match.\n");
+    if (count > 1)
+        ok(compare_media_types(mts[1], source_mt), "Media types didn't match.\n");
+    CoTaskMemFree(mts[0]);
+    if (count > 1)
+        CoTaskMemFree(mts[1]);
+    IEnumMediaTypes_Release(enummt);
 
-    hr = CoCreateInstance(&CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, &IID_IGraphBuilder,
-            (LPVOID*)&graphBuilder);
-    ok(SUCCEEDED(hr), "couldn't create graph builder, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    hr = IPin_QueryAccept(source, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 
-    hr = ICaptureGraphBuilder2_SetFiltergraph(captureGraphBuilder, graphBuilder);
-    ok(SUCCEEDED(hr), "ICaptureGraphBuilder2_SetFilterGraph() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    req_mt.lSampleSize = 2;
+    req_mt.bFixedSizeSamples = TRUE;
+    hr = IPin_QueryAccept(source, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 
-    hr = CoCreateInstance(&CLSID_NullRenderer, NULL, CLSCTX_INPROC_SERVER,
-            &IID_IBaseFilter, (LPVOID*)&nullRenderer);
-    ok(SUCCEEDED(hr) ||
-            /* Windows 2008: http://stackoverflow.com/questions/29410348/initialize-nullrender-failed-with-error-regdb-e-classnotreg-on-win2008-r2 */
-            broken(hr == REGDB_E_CLASSNOTREG), "couldn't create NullRenderer, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
-    hr = IGraphBuilder_AddFilter(graphBuilder, nullRenderer, NULL);
-    ok(SUCCEEDED(hr), "IGraphBuilder_AddFilter() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    req_mt.cbFormat = sizeof(count);
+    req_mt.pbFormat = (BYTE *)&count;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.cbFormat = 0;
+    req_mt.pbFormat = NULL;
 
-    audioSource = create_audio_SourceFilter();
-    ok(audioSource != NULL, "couldn't create audio source\n");
-    if (audioSource == NULL)
-        goto end;
-    hr = IGraphBuilder_AddFilter(graphBuilder, &audioSource->IBaseFilter_iface, NULL);
-    ok(SUCCEEDED(hr), "IGraphBuilder_AddFilter() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    req_mt.majortype = MEDIATYPE_Audio;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.majortype = GUID_NULL;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.majortype = MEDIATYPE_Stream;
 
-    test_function(captureGraphBuilder, graphBuilder, audioSource, nullRenderer);
+    req_mt.subtype = MEDIASUBTYPE_PCM;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.subtype = GUID_NULL;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.subtype = MEDIASUBTYPE_Avi;
 
-end:
-    if (nullRenderer)
-        IBaseFilter_Release(nullRenderer);
-    if (audioSource)
-        IBaseFilter_Release(&audioSource->IBaseFilter_iface);
-    if (captureGraphBuilder)
-        ICaptureGraphBuilder2_Release(captureGraphBuilder);
-    if (graphBuilder)
-        IGraphBuilder_Release(graphBuilder);
+    req_mt.formattype = FORMAT_WaveFormatEx;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.formattype = GUID_NULL;
+    hr = IPin_QueryAccept(source, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    req_mt.formattype = FORMAT_None;
+
+    req_mt.majortype = MEDIATYPE_Audio;
+    req_mt.subtype = MEDIASUBTYPE_PCM;
+    req_mt.formattype = test_iid;
+    req_mt.cbFormat = sizeof(count);
+    req_mt.pbFormat = (BYTE *)&count;
+    req_mt.bTemporalCompression = TRUE;
+    hr = IPin_QueryAccept(source, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 }
 
-static void test_video_smart_tee_filter_auto_insertion(
-        void (*test_function)(ICaptureGraphBuilder2 *cgb, IGraphBuilder *gb,
-                SourceFilter *videoSource, IBaseFilter *nullRenderer))
+static void test_source_connection(AM_MEDIA_TYPE req_mt, IFilterGraph2 *graph,
+        struct testfilter *testsink, IPin *source)
 {
+    const AM_MEDIA_TYPE sink_mt = req_mt;
+    AM_MEDIA_TYPE mt;
     HRESULT hr;
-    ICaptureGraphBuilder2 *captureGraphBuilder = NULL;
-    IGraphBuilder *graphBuilder = NULL;
-    IBaseFilter *nullRenderer = NULL;
-    SourceFilter *videoSource = NULL;
+    IPin *peer;
 
-    hr = CoCreateInstance(&CLSID_CaptureGraphBuilder2, NULL, CLSCTX_INPROC_SERVER,
-            &IID_ICaptureGraphBuilder2, (void**)&captureGraphBuilder);
-    ok(SUCCEEDED(hr), "couldn't create capture graph builder, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    peer = (IPin *)0xdeadbeef;
+    hr = IPin_ConnectedTo(source, &peer);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+    ok(!peer, "Got peer %p.\n", peer);
 
-    hr = CoCreateInstance(&CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, &IID_IGraphBuilder,
-            (LPVOID*)&graphBuilder);
-    ok(SUCCEEDED(hr), "couldn't create graph builder, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    hr = IPin_ConnectionMediaType(source, &mt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
 
-    hr = ICaptureGraphBuilder2_SetFiltergraph(captureGraphBuilder, graphBuilder);
-    ok(SUCCEEDED(hr), "ICaptureGraphBuilder2_SetFilterGraph() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    /* Exact connection. */
 
-    hr = CoCreateInstance(&CLSID_NullRenderer, NULL, CLSCTX_INPROC_SERVER,
-            &IID_IBaseFilter, (LPVOID*)&nullRenderer);
-    ok(SUCCEEDED(hr) ||
-            /* Windows 2008: http://stackoverflow.com/questions/29410348/initialize-nullrender-failed-with-error-regdb-e-classnotreg-on-win2008-r2 */
-            broken(hr == REGDB_E_CLASSNOTREG), "couldn't create NullRenderer, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
-    hr = IGraphBuilder_AddFilter(graphBuilder, nullRenderer, NULL);
-    ok(SUCCEEDED(hr), "IGraphBuilder_AddFilter() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 
-    videoSource = create_video_SourceFilter();
-    ok(videoSource != NULL, "couldn't create audio source\n");
-    if (videoSource == NULL)
-        goto end;
-    hr = IGraphBuilder_AddFilter(graphBuilder, &videoSource->IBaseFilter_iface, NULL);
-    ok(SUCCEEDED(hr), "IGraphBuilder_AddFilter() failed, hr=0x%08x\n", hr);
-    if (FAILED(hr))
-        goto end;
+    hr = IPin_ConnectedTo(source, &peer);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(peer == &testsink->sink.pin.IPin_iface, "Got peer %p.\n", peer);
+    IPin_Release(peer);
 
-    test_function(captureGraphBuilder, graphBuilder, videoSource, nullRenderer);
+    hr = IPin_ConnectionMediaType(source, &mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&mt, &req_mt), "Media types didn't match.\n");
+    ok(compare_media_types(&testsink->sink.pin.mt, &req_mt), "Media types didn't match.\n");
 
-end:
-    if (nullRenderer)
-        IBaseFilter_Release(nullRenderer);
-    if (videoSource)
-        IBaseFilter_Release(&videoSource->IBaseFilter_iface);
-    if (captureGraphBuilder)
-        ICaptureGraphBuilder2_Release(captureGraphBuilder);
-    if (graphBuilder)
-        IGraphBuilder_Release(graphBuilder);
+    hr = IFilterGraph2_Disconnect(graph, source);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IFilterGraph2_Disconnect(graph, source);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(testsink->sink.pin.peer == source, "Got peer %p.\n", testsink->sink.pin.peer);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.subtype = GUID_NULL;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    todo_wine ok(hr == VFW_E_TYPE_NOT_ACCEPTED, "Got hr %#x.\n", hr);
+    if (hr == S_OK)
+    {
+        IFilterGraph2_Disconnect(graph, source);
+        IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+    }
+    req_mt.subtype = sink_mt.subtype;
+
+    req_mt.majortype = MEDIATYPE_Audio;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    todo_wine ok(hr == VFW_E_TYPE_NOT_ACCEPTED, "Got hr %#x.\n", hr);
+    if (hr == S_OK)
+    {
+        IFilterGraph2_Disconnect(graph, source);
+        IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+    }
+
+    /* Connection with wildcards. */
+
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &sink_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.majortype = GUID_NULL;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &sink_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.subtype = MEDIASUBTYPE_RGB32;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == VFW_E_NO_ACCEPTABLE_TYPES, "Got hr %#x.\n", hr);
+
+    req_mt.subtype = GUID_NULL;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &sink_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.formattype = FORMAT_WaveFormatEx;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == VFW_E_NO_ACCEPTABLE_TYPES, "Got hr %#x.\n", hr);
+
+    req_mt = sink_mt;
+    req_mt.formattype = GUID_NULL;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &sink_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.subtype = MEDIASUBTYPE_RGB32;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == VFW_E_NO_ACCEPTABLE_TYPES, "Got hr %#x.\n", hr);
+
+    req_mt.subtype = GUID_NULL;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &sink_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    req_mt.majortype = MEDIATYPE_Audio;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, &req_mt);
+    ok(hr == VFW_E_NO_ACCEPTABLE_TYPES, "Got hr %#x.\n", hr);
+
+    testsink->sink_mt = &req_mt;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, NULL);
+    todo_wine ok(hr == VFW_E_NO_ACCEPTABLE_TYPES, "Got hr %#x.\n", hr);
+    if (hr == S_OK)
+    {
+        IFilterGraph2_Disconnect(graph, source);
+        IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+    }
+
+    req_mt = sink_mt;
+    req_mt.lSampleSize = 3;
+    hr = IFilterGraph2_ConnectDirect(graph, source, &testsink->sink.pin.IPin_iface, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&testsink->sink.pin.mt, &req_mt), "Media types didn't match.\n");
+    IFilterGraph2_Disconnect(graph, source);
+    IFilterGraph2_Disconnect(graph, &testsink->sink.pin.IPin_iface);
+
+    testsink->sink_mt = NULL;
+}
+
+static void test_connect_pin(void)
+{
+    AM_MEDIA_TYPE req_mt =
+    {
+        .majortype = MEDIATYPE_Stream,
+        .subtype = MEDIASUBTYPE_Avi,
+        .formattype = FORMAT_None,
+        .lSampleSize = 1,
+    };
+    IBaseFilter *filter = create_smart_tee();
+    struct testfilter testsource, testsink;
+    IPin *sink, *capture, *preview, *peer;
+    AM_MEDIA_TYPE mt, *mts[3];
+    IEnumMediaTypes *enummt;
+    IFilterGraph2 *graph;
+    HRESULT hr;
+    ULONG ref;
+
+    testfilter_init(&testsource);
+    testfilter_init(&testsink);
+    CoCreateInstance(&CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
+            &IID_IFilterGraph2, (void **)&graph);
+    IFilterGraph2_AddFilter(graph, &testsource.filter.IBaseFilter_iface, L"source");
+    IFilterGraph2_AddFilter(graph, &testsink.filter.IBaseFilter_iface, L"sink");
+    IFilterGraph2_AddFilter(graph, filter, L"sample grabber");
+    IBaseFilter_FindPin(filter, L"Input", &sink);
+    IBaseFilter_FindPin(filter, L"Capture", &capture);
+    IBaseFilter_FindPin(filter, L"Preview", &preview);
+
+    testsource.source_mt.majortype = MEDIATYPE_Video;
+    testsource.source_mt.subtype = MEDIASUBTYPE_RGB8;
+    testsource.source_mt.formattype = FORMAT_VideoInfo;
+
+    hr = IPin_EnumMediaTypes(sink, &enummt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IEnumMediaTypes_Next(enummt, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    IEnumMediaTypes_Release(enummt);
+
+    hr = IPin_EnumMediaTypes(capture, &enummt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+    hr = IPin_EnumMediaTypes(preview, &enummt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+
+    hr = IPin_QueryAccept(sink, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IPin_QueryAccept(capture, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    hr = IPin_QueryAccept(preview, &req_mt);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    /* Test sink connection. */
+
+    peer = (IPin *)0xdeadbeef;
+    hr = IPin_ConnectedTo(sink, &peer);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+    ok(!peer, "Got peer %p.\n", peer);
+
+    hr = IPin_ConnectionMediaType(sink, &mt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+
+    hr = IFilterGraph2_ConnectDirect(graph, &testsource.source.pin.IPin_iface, sink, &req_mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IPin_ConnectedTo(sink, &peer);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(peer == &testsource.source.pin.IPin_iface, "Got peer %p.\n", peer);
+    IPin_Release(peer);
+
+    hr = IPin_ConnectionMediaType(sink, &mt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(&mt, &req_mt), "Media types didn't match.\n");
+
+    hr = IPin_EnumMediaTypes(sink, &enummt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IEnumMediaTypes_Next(enummt, 1, mts, NULL);
+    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    IEnumMediaTypes_Release(enummt);
+
+    test_source_media_types(req_mt, &testsource.source_mt, capture);
+    test_source_media_types(req_mt, &testsource.source_mt, preview);
+    test_source_connection(req_mt, graph, &testsink, capture);
+    test_source_connection(req_mt, graph, &testsink, preview);
+
+    hr = IFilterGraph2_Disconnect(graph, sink);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IFilterGraph2_Disconnect(graph, sink);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(testsource.source.pin.peer == sink, "Got peer %p.\n", testsource.source.pin.peer);
+    IFilterGraph2_Disconnect(graph, &testsource.source.pin.IPin_iface);
+
+    peer = (IPin *)0xdeadbeef;
+    hr = IPin_ConnectedTo(sink, &peer);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+    ok(!peer, "Got peer %p.\n", peer);
+
+    hr = IPin_ConnectionMediaType(sink, &mt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
+
+    IPin_Release(sink);
+    IPin_Release(capture);
+    IPin_Release(preview);
+    ref = IFilterGraph2_Release(graph);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+    ref = IBaseFilter_Release(filter);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+    ref = IBaseFilter_Release(&testsource.filter.IBaseFilter_iface);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+    ref = IBaseFilter_Release(&testsink.filter.IBaseFilter_iface);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
 }
 
 START_TEST(smartteefilter)
@@ -2519,15 +2776,10 @@ START_TEST(smartteefilter)
     test_pin_info();
     test_enum_media_types();
     test_unconnected_filter_state();
+    test_connect_pin();
 
     test_smart_tee_filter();
 
-    test_audio_smart_tee_filter_auto_insertion(test_audio_preview);
-    test_audio_smart_tee_filter_auto_insertion(test_audio_capture);
-
-    test_video_smart_tee_filter_auto_insertion(test_video_preview);
-    test_video_smart_tee_filter_auto_insertion(test_video_capture);
-
     CloseHandle(event);
     CoUninitialize();
 }
