@@ -1665,7 +1665,6 @@ static void init_default_render_states(DWORD rs[WINEHIGHEST_RENDER_STATE + 1], c
     rs[WINED3D_RS_FOGEND] = tmpfloat.d;
     tmpfloat.f = 1.0f;
     rs[WINED3D_RS_FOGDENSITY] = tmpfloat.d;
-    rs[WINED3D_RS_EDGEANTIALIAS] = FALSE;
     rs[WINED3D_RS_RANGEFOGENABLE] = FALSE;
     rs[WINED3D_RS_STENCILENABLE] = FALSE;
     rs[WINED3D_RS_STENCILFAIL] = WINED3D_STENCIL_OP_KEEP;
@@ -1819,8 +1818,8 @@ static void state_init_default(struct wined3d_state *state, const struct wined3d
     TRACE("state %p, d3d_info %p.\n", state, d3d_info);
 
     get_identity_matrix(&identity);
-    state->gl_primitive_type = ~0u;
-    state->gl_patch_vertices = 0;
+    state->primitive_type = WINED3D_PT_UNDEFINED;
+    state->patch_vertex_count = 0;
 
     /* Set some of the defaults for lights, transforms etc */
     state->transforms[WINED3D_TS_PROJECTION] = identity;
