@@ -104,9 +104,11 @@ typedef enum _CONFIGURATION_TYPE
     DockingInformation,
     RealModeIrqRoutingTable,
     RealModePCIEnumeration,
-    MaximunType
+    MaximumType
 } CONFIGURATION_TYPE, *PCONFIGURATION_TYPE;
 
+#define IMAGE_ADDRESSING_MODE_32BIT 3
+
 typedef struct _IMAGE_INFO
 {
     union
@@ -118,7 +120,11 @@ typedef struct _IMAGE_INFO
             ULONG SystemModeImage      : 1;
             ULONG ImageMappedToAllPids : 1;
             ULONG ExtendedInfoPresent  : 1;
-            ULONG Reserved             : 21;
+            ULONG MachineTypeMismatch : 1;
+            ULONG ImageSignatureLevel : 4;
+            ULONG ImageSignatureType : 3;
+            ULONG ImagePartialMap : 1;
+            ULONG Reserved : 12;
         } DUMMYSTRUCTNAME;
     } DUMMYUNIONNAME;
     PVOID  ImageBase;
@@ -214,15 +220,19 @@ typedef GUID UUID;
 NTSTATUS  WINAPI ExUuidCreate(UUID*);
 NTSTATUS  WINAPI IoQueryDeviceDescription(PINTERFACE_TYPE,PULONG,PCONFIGURATION_TYPE,PULONG,
                                   PCONFIGURATION_TYPE,PULONG,PIO_QUERY_DEVICE_ROUTINE,PVOID);
+void      WINAPI IoRegisterBootDriverReinitialization(DRIVER_OBJECT*,PDRIVER_REINITIALIZE,void*);
 void      WINAPI IoRegisterDriverReinitialization(PDRIVER_OBJECT,PDRIVER_REINITIALIZE,PVOID);
 NTSTATUS  WINAPI IoRegisterShutdownNotification(PDEVICE_OBJECT);
 BOOLEAN   WINAPI KeAreApcsDisabled(void);
+void      WINAPI DECLSPEC_NORETURN KeBugCheck(ULONG);
 NTSTATUS  WINAPI KeExpandKernelStackAndCallout(PEXPAND_STACK_CALLOUT,void*,SIZE_T);
 void      WINAPI KeSetTargetProcessorDpc(PRKDPC,CCHAR);
 BOOLEAN   WINAPI MmIsAddressValid(void *);
 HANDLE    WINAPI PsGetProcessId(PEPROCESS);
+void *    WINAPI PsGetProcessSectionBaseAddress(PEPROCESS);
 HANDLE    WINAPI PsGetThreadId(PETHREAD);
 HANDLE    WINAPI PsGetThreadProcessId(PETHREAD);
+NTSTATUS  WINAPI PsRemoveLoadImageNotifyRoutine(PLOAD_IMAGE_NOTIFY_ROUTINE);
 NTSTATUS  WINAPI PsSetCreateProcessNotifyRoutine(PCREATE_PROCESS_NOTIFY_ROUTINE,BOOLEAN);
 NTSTATUS  WINAPI PsSetCreateProcessNotifyRoutineEx(PCREATE_PROCESS_NOTIFY_ROUTINE_EX,BOOLEAN);
 NTSTATUS  WINAPI PsSetCreateThreadNotifyRoutine(PCREATE_THREAD_NOTIFY_ROUTINE);
