@@ -47,6 +47,7 @@ int link_ext_symbols = 0;
 int force_pointer_size = 0;
 int unwind_tables = 0;
 int unix_lib = 0;
+int safe_seh = 0;
 
 #ifdef __i386__
 enum target_cpu target_cpu = CPU_x86;
@@ -115,6 +116,7 @@ enum exec_mode_values
     MODE_IMPLIB,
     MODE_STATICLIB,
     MODE_BUILTIN,
+    MODE_FIXUP_CTORS,
     MODE_RESOURCES
 };
 
@@ -289,6 +291,7 @@ static const char usage_str[] =
 "   -N, --dll-name=DLLNAME    Set the DLL name (default: from input file name)\n"
 "   -o, --output=NAME         Set the output file name (default: stdout)\n"
 "   -r, --res=RSRC.RES        Load resources from RSRC.RES\n"
+"       --safeseh             Mark object files as SEH compatible\n"
 "       --save-temps          Do not delete the generated intermediate files\n"
 "       --subsystem=SUBSYS    Set the subsystem (one of native, windows, console, wince)\n"
 "   -u, --undefined=SYMBOL    Add an undefined reference to SYMBOL when linking\n"
@@ -301,8 +304,9 @@ static const char usage_str[] =
 "       --exe                 Build an executable from object files\n"
 "       --implib              Build an import library\n"
 "       --staticlib           Build a static library\n"
-"       --builtin             Mark a library as a Wine builtin\n"
 "       --resources           Build a .o or .res file for the resource files\n\n"
+"       --builtin             Mark a library as a Wine builtin\n"
+"       --fixup-ctors         Fixup the constructors data after the module has been built\n"
 "The mode options are mutually exclusive; you must specify one and only one.\n\n";
 
 enum long_options_values
@@ -316,11 +320,13 @@ enum long_options_values
     LONG_OPT_CCCMD,
     LONG_OPT_EXTERNAL_SYMS,
     LONG_OPT_FAKE_MODULE,
+    LONG_OPT_FIXUP_CTORS,
     LONG_OPT_LARGE_ADDRESS_AWARE,
     LONG_OPT_LDCMD,
     LONG_OPT_NMCMD,
     LONG_OPT_NXCOMPAT,
     LONG_OPT_RESOURCES,
+    LONG_OPT_SAFE_SEH,
     LONG_OPT_SAVE_TEMPS,
     LONG_OPT_STATICLIB,
     LONG_OPT_SUBSYSTEM,
@@ -341,11 +347,13 @@ static const struct option long_options[] =
     { "cc-cmd",        1, 0, LONG_OPT_CCCMD },
     { "external-symbols", 0, 0, LONG_OPT_EXTERNAL_SYMS },
     { "fake-module",   0, 0, LONG_OPT_FAKE_MODULE },
+    { "fixup-ctors",   0, 0, LONG_OPT_FIXUP_CTORS },
     { "large-address-aware", 0, 0, LONG_OPT_LARGE_ADDRESS_AWARE },
     { "ld-cmd",        1, 0, LONG_OPT_LDCMD },
     { "nm-cmd",        1, 0, LONG_OPT_NMCMD },
     { "nxcompat",      1, 0, LONG_OPT_NXCOMPAT },
     { "resources",     0, 0, LONG_OPT_RESOURCES },
+    { "safeseh",       0, 0, LONG_OPT_SAFE_SEH },
     { "save-temps",    0, 0, LONG_OPT_SAVE_TEMPS },
     { "subsystem",     1, 0, LONG_OPT_SUBSYSTEM },
     { "version",       0, 0, LONG_OPT_VERSION },
@@ -385,8 +393,8 @@ static void set_exec_mode( enum exec_mode_values mode )
 /* get the default entry point for a given spec file */
 static const char *get_default_entry_point( const DLLSPEC *spec )
 {
-    if (spec->characteristics & IMAGE_FILE_DLL) return "__wine_spec_dll_entry";
-    if (spec->subsystem == IMAGE_SUBSYSTEM_NATIVE) return "__wine_spec_drv_entry";
+    if (spec->characteristics & IMAGE_FILE_DLL) return "DllMain";
+    if (spec->subsystem == IMAGE_SUBSYSTEM_NATIVE) return "DriverEntry";
     if (spec->type == SPEC_WIN16) return "__wine_spec_exe16_entry";
     return "__wine_spec_exe_entry";
 }
@@ -511,6 +519,9 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
         case LONG_OPT_BUILTIN:
             set_exec_mode( MODE_BUILTIN );
             break;
+        case LONG_OPT_FIXUP_CTORS:
+            set_exec_mode( MODE_FIXUP_CTORS );
+            break;
         case LONG_OPT_ASCMD:
             as_command = strarray_fromstring( optarg, " " );
             break;
@@ -536,6 +547,9 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
             if (optarg[0] == 'n' || optarg[0] == 'N')
                 spec->dll_characteristics &= ~IMAGE_DLLCHARACTERISTICS_NX_COMPAT;
             break;
+        case LONG_OPT_SAFE_SEH:
+            safe_seh = 1;
+            break;
         case LONG_OPT_RESOURCES:
             set_exec_mode( MODE_RESOURCES );
             break;
@@ -701,6 +715,10 @@ int main(int argc, char **argv)
         if (!argv[0]) fatal_error( "missing file argument for --builtin option\n" );
         make_builtin_files( argv );
         break;
+    case MODE_FIXUP_CTORS:
+        if (!argv[0]) fatal_error( "missing file argument for --fixup-ctors option\n" );
+        fixup_constructors( argv );
+        break;
     case MODE_RESOURCES:
         load_resources( argv, spec );
         output_res_o_file( spec );
